@@ -2,18 +2,28 @@ import pandas as pd
 import numpy as np
 import copy
 import json
+import os
 from io import BytesIO
+import xlsxwriter
+import yaml
+
+settings = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'settings.yaml')
+with open(settings, 'r') as f:
+    settings = yaml.load(f.read(),Loader=yaml.SafeLoader)
 
 
 class _Workbook:
     """
-    Excel Workbook level configurations.  This is not part of the end_user API
+    Excel Workbook level configurations.  This is not part of the end_user API.
+    This class  facilitates:
+    1. Crawling the xlcompose object on which `to_excel` is called
+    2. Writing the each nested object to an Excel file
+
     """
-    max_column_width = 30
-    max_portrait_width = 120
-    footer = '&CPage &P of &N\n&A'
 
     def __init__(self, workbook_path, exhibits, default_formats):
+        """ Initialize the writer object
+        """
         self.formats = {}
         self.writer = pd.ExcelWriter(workbook_path)
         self.exhibits = exhibits
@@ -41,13 +51,30 @@ class _Workbook:
         self.writer.close()
 
     def _write(self, exhibit, sheet, start_row=0, start_col=0):
+        """
+        Parameters
+        ----------
+        exhibit :
+            An xlcompose object
+        sheet : str
+            The sheet name in Excel to write to
+        start_row : int
+            The starting row on which to write the exhibit
+        start_col : int
+            The starting column on which to write the exhibit
+        """
+        # Get xlcompose object for special handling
         klass = exhibit.__class__.__name__
         if getattr(exhibit, 'title', None) is not None:
+            ## Special handling of title.  It must live in a Column
+            #  if it doesn't already
             t = copy.deepcopy(exhibit.title)
             exhibit.title = None
             exhibit = Column(t, exhibit)
             self._write(exhibit, sheet, start_row, start_col)
         elif klass in ['Row', 'Column']:
+            ## Need to render each object in Row and Column args keeping in
+            #  mind the start_row and start_col of the container
             start_row = start_row
             start_col = start_col
             for item in exhibit.args:
@@ -57,14 +84,17 @@ class _Workbook:
                 if klass == 'Row':
                    start_col = start_col + item.width
         else:
+            # General rendering for Non-container objects
             exhibit.start_row = start_row
             exhibit.start_col = start_col
             exhibit.sheet_name = sheet
+            # Create sheet if it doesn't already exist
             try:
                 exhibit.worksheet = self.writer.sheets[exhibit.sheet_name]
             except:
                 pd.DataFrame().to_excel(self.writer, sheet_name=exhibit.sheet_name)
                 exhibit.worksheet = self.writer.sheets[exhibit.sheet_name]
+
             if klass in ['DataFrame', 'RSpacer', 'CSpacer']:
                 if exhibit.header:
                     self._write_header(exhibit)
@@ -73,31 +103,53 @@ class _Workbook:
                 self._register_formats(exhibit)
                 self._write_data(exhibit)
             if klass in ['Title', 'Series']:
-                self._write_title(exhibit)
+                self._write_series(exhibit)
             if klass == 'Image':
                 self._write_image(exhibit)
 
     def _set_worksheet_properties(self, exhibit, sheet):
-        ''' Format column widths, headers footers, etc.'''
+        """ Set worksheet level properties. Called once the entire sheet has
+        been rendered. These set worksheet level settings and in many cases is
+        a straight pass through to xlsxwriter.  Can we inspect the xlsxwriter
+        properties to dynamically support its functionality?
+
+        Parameters
+        ----------
+        exhibit :
+            An xlcompose object
+        sheet : str
+            The sheet name in Excel to write to
+        """
         exhibit.worksheet = self.writer.sheets[sheet]
-        widths = [min(self.max_column_width, item)
+        widths = [min(settings['max_column_width'], item)
                   for item in exhibit.column_widths]
+        heights = [min(settings['max_row_height'], item) if item is not None else item
+                   for item in exhibit.row_heights]
         for num, item in enumerate(widths):
             exhibit.worksheet.set_column(num, num, item)
+        for num, item in enumerate(heights):
+            if item is not None:
+                exhibit.worksheet.set_row(num, item)
         kwargs = exhibit.kwargs
         exhibit.worksheet.fit_to_pages(*kwargs.get('fit_to_pages', (1,0)))
-        if kwargs.get('freeze_panes'):
-            exhibit.worksheet.freeze_panes(*kwargs['freeze_panes'])
-        if kwargs.get('set_page_view'):
-            exhibit.worksheet.set_page_view()
-        if kwargs.get('print_row_col_headers'):
-            exhibit.worksheet.print_row_col_headers()
-        if kwargs.get('hide_row_col_headers'):
-            exhibit.worksheet.hide_row_col_headers()
-        if kwargs.get('center_vertically'):
-            exhibit.worksheet.center_vertically()
-        if kwargs.get('center_horizontally'):
-            exhibit.worksheet.center_horizontally()
+        bool_funcs = [
+            'set_page_view', 'print_row_col_headers', 'hide_row_col_headers',
+            'center_vertically', 'center_horizontally']
+        for func in bool_funcs:
+            if kwargs.get(func):
+                getattr(exhibit.worksheet, func)()
+        passthru_funcs = [
+            'hide_gridlines', 'set_print_scale', 'set_start_page', 'set_paper'
+            'set_h_pagebreaks', 'set_v_pagebreaks', 'print_across']
+        for func in passthru_funcs:
+            if kwargs.get(func):
+                getattr(exhibit.worksheet, func)(kwargs[func])
+        starg_funcs = [
+            'freeze_panes', 'repeat_rows', 'repeat_columns', 'set_margins',
+            'print_area']
+        for func in starg_funcs:
+            if kwargs.get(func):
+                getattr(exhibit.worksheet, func)(*kwargs[func])
         if kwargs.get('set_header', None) is not None:
             if type(kwargs['set_header']) is list:
                 exhibit.worksheet.set_header('\n'.join(kwargs['set_header']))
@@ -108,39 +160,21 @@ class _Workbook:
                 exhibit.worksheet.set_footer('\n'.join(kwargs['set_footer']))
             else:
                 exhibit.worksheet.set_footer(kwargs['set_footer'])
-        if kwargs.get('repeat_rows', None) is not None:
-            exhibit.worksheet.repeat_rows(*kwargs['repeat_rows'])
-        if kwargs.get('repeat_columns', None) is not None:
-            exhibit.worksheet.repeat_columns(*kwargs['repeat_columns'])
-        if kwargs.get('set_margins', None) is not None:
-            exhibit.worksheet.set_margins(*kwargs['set_margins'])
-        if kwargs.get('hide_gridlines', None) is not None:
-            exhibit.worksheet.hide_gridlines(kwargs['hide_gridlines'])
-        if kwargs.get('set_print_scale'):
-            exhibit.worksheet.set_print_scale(kwargs['set_print_scale'])
-        if kwargs.get('set_start_page'):
-            exhibit.worksheet.set_start_page(kwargs['set_start_page'])
-        if kwargs.get('set_h_pagebreaks'):
-            exhibit.worksheet.set_h_pagebreaks(kwargs['set_h_pagebreaks'])
-        if kwargs.get('set_v_pagebreaks'):
-            exhibit.worksheet.set_v_pagebreaks(kwargs['set_v_pagebreaks'])
-        if kwargs.get('print_across'):
-            exhibit.worksheet.print_across(kwargs['print_across'])
-        if kwargs.get('print_area'):
-            exhibit.worksheet.print_area(*kwargs['print_area'])
-        if kwargs.get('set_paper', None) is not None:
-            exhibit.worksheet.set_paper(kwargs['set_paper'])
         if kwargs.get('set_landscape'):
             exhibit.worksheet.set_landscape()
         elif kwargs.get('set_portrait'):
             exhibit.worksheet.set_portrait()
         else:
-            if sum(widths) > self.max_portrait_width:
+            if sum(widths) > settings['max_portrait_width']:
                 exhibit.worksheet.set_landscape()
             else:
                 exhibit.worksheet.set_portrait()
 
-    def _write_title(self, exhibit):
+    def _write_series(self, exhibit):
+        """ Writes a Series or Title object.  Special considerations are
+        that these objects can take a format list that applies to each element
+        of the Series.  These also merge cells to span their designated `width`.
+        """
         start_row = exhibit.start_row
         start_col = exhibit.start_col
         end_row = start_row + exhibit.height
@@ -165,6 +199,9 @@ class _Workbook:
                     title_format[r - start_row])
 
     def _write_image(self, exhibit):
+        """ Writes an image object and merges the cells behind it based on the
+        images designated `height` and `width`
+        """
         exhibit.worksheet.insert_image(
             exhibit.start_row, exhibit.start_col, exhibit.data,
             options=exhibit.formats)
@@ -175,10 +212,12 @@ class _Workbook:
 
     def _write_header(self, exhibit):
         ''' Adds column headers to data table '''
-        if not exhibit.index:
-            headers = exhibit.data.columns
+        if type(exhibit.data.columns) == pd.PeriodIndex:
+            header = exhibit.data.columns.astype(str)
         else:
-            headers = [exhibit.index_label]+list(exhibit.data.columns)
+            header = exhibit.data.columns
+        if exhibit.index:
+            headers = [exhibit.index_label]+list(header)
         header_format = self.default_formats.copy()
         header_format.update(exhibit.header_formats)
         header_format = self.writer.book.add_format(header_format)
@@ -252,62 +291,10 @@ class _Workbook:
                         width=exhibit.column_widths[c_idx + exhibit.index])
 
 
-class Title:
-    """ Make cool looking titles
-
-    Parameters
-    ----------
-    data : str or list of str
-        Title and subtitle texts
-    formats : list of dicts
-        Formats to be applied to the title
-    width :
-        The width the title should span
-    """
-    def __init__(self, data, formats=[], width=None, column_widths=None, *args, **kwargs):
-        if type(data) is str:
-            data = [data]
-        self.data = pd.DataFrame(data)
-        self.title_formats = self._set_format(formats)
-        self.width = width
-        self.height = len(self.data)
-        self.header = False
-        self.index = False
-        self.col_nums = False
-        self.formats = {}
-        if column_widths is not None and width is not None:
-            self._column_widths = [column_widths]*self.width
-        self.kwargs = kwargs
-
-    @property
-    def column_widths(self):
-        if hasattr(self, '_column_widths'):
-            return self._column_widths
-        return [0]*self.width
-
-    @column_widths.setter
-    def column_widths(self, value):
-        self._column_widths = value
-
-    def __len__(self):
-        return len(self.data)
-
-    def _default_format(self):
-        return [{'font_size': 20, 'align': 'center'},
-                {'font_size': 16, 'align': 'center'},
-                {'font_size': 16, 'align': 'center'}] + \
-               [{'font_size': 13, 'align': 'center'}] * (len(self.data)-3)
-
-    def _set_format(self, overlay):
-        original = self._default_format()
-        if overlay is not None:
-            if type(overlay) is list:
-                for num, item in enumerate(overlay):
-                    original[num].update(item)
-            else:
-                for num, item in enumerate(original):
-                    original[num].update(overlay)
-        return original
+class _XLCBase:
+    px_per_row = 15
+    with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'styles.css'), 'r') as f:
+        styles = '<style>' + f.read() + '</style>'
 
     def to_excel(self, workbook_path, default_formats=None):
         """ Outputs object to Excel.
@@ -320,33 +307,151 @@ class Title:
         _Workbook(workbook_path=workbook_path, exhibits=self,
                   default_formats=default_formats).to_excel()
 
-class Series(Title):
-    def __init__(self, data, formats=[], width=1, column_widths=1):
-        data = pd.Series(data).to_frame()
-        super().__init__(data, formats, width, column_widths)
+    def _repr_html_(self):
+        return self.styles + self._get_html()
+
+
+    def _get_html(self, my_height=0, my_width=100):
+        width = 'width:' + str(my_width) + '%;' if my_width < 100 else 'width: auto;'
+        name = self.__class__.__name__
+        return '<div class="xlccontainer-' + name + \
+                '" style="height:' + str(self.height*self.px_per_row) + 'px;' + \
+                width + '"><div class="xlclabel-' + \
+                name + '">' + name + \
+                '</div></div>\n'
+
+
+class Title(_XLCBase):
+    """ Title objects are Series-like objects that has its own formatting style.
+
+    Difference between a Series and a Title is that a Title `width` will be
+    inferred from its container.  Additionally, Title objects have a different
+    default format.
+
+    Parameters
+    ----------
+    data : str or list of str
+        Title and subtitle texts
+    formats : list
+        The formats to be applied to the Title. Each element in the Title
+        will be assigned a format from this list with the corresponding index.
+    width :
+        The width the title should span.  If omitted, the title will take on
+        the width of its container.
+    column_widths : list
+        list of floats representing the column widths of each column within the
+        DataFrame.  If omitted, then widths are set by inspecting the data.
+    row_heights : list
+        list of floats representing the row heights of each row within the
+        Series.  If omitted, then heights are set by inspecting the data.
+    """
+
+    def __init__(self, data, formats=[], width=None,
+                 column_widths=None, row_heights=None, *args, **kwargs):
+        if type(data) is str:
+            data = [data]
+        self.data = pd.DataFrame(data)
+        self.title_formats = self._set_format(formats)
+        self.width = width
+        self.height = len(self.data)
+        self.header = False
+        self.index = False
+        self.col_nums = False
+        self.formats = {}
+        if column_widths is not None and width is not None:
+            self._column_widths = [column_widths]*self.width
+        if row_heights is not None:
+            self._row_heights = [row_heights]*len(self.data)
+        self.kwargs = kwargs
+
+    @property
+    def column_widths(self):
+        if hasattr(self, '_column_widths'):
+            return self._column_widths
+        return [0]*self.width
+
+    @column_widths.setter
+    def column_widths(self, value):
+        self._column_widths = value
+
+    @property
+    def row_heights(self):
+        if hasattr(self, '_row_heights'):
+            return self._row_heights
+        return [None] * len(self.data)
+
+    @row_heights.setter
+    def row_heights(self, value):
+        self._row_heights = value
+
+    def __len__(self):
+        return len(self.data)
 
     def _default_format(self):
-        base_formats = {
-            'float64': {'num_format': '#,0.00', 'align': 'center'},
-            'float32': {'num_format': '#,0.00', 'align': 'center'},
-            'int64': {'num_format': '#,0', 'align': 'center'},
-            'int32': {'num_format': '#,0', 'align': 'center'},
-            '<M8[ns]': {'num_format': 'yyyy-mm-dd hh:mm', 'align': 'center'},
-            'datetime64[ns]': {'num_format': 'yyyy-mm-dd hh:mm', 'align': 'center'},
-            'object': {'align': 'left'},
-        }
+        title_formats = copy.deepcopy(settings['title_formats'])
+        return title_formats[:3] + title_formats[-1:] * (len(self.data)-3)
+
+    def _set_format(self, overlay):
+        original = self._default_format()
+        if overlay is not None:
+            if type(overlay) is list:
+                for num, item in enumerate(overlay):
+                    original[num].update(item)
+            else:
+                for num, item in enumerate(original):
+                    original[num].update(overlay)
+        return original
+
+
+class Series(Title):
+    """ Excel-ready Series object. This component does not render the index or
+    series name.  To include those, see the `DataFrame` component.
+
+    Parameters
+    ----------
+    data : list or Series
+        The data to be placed into the series
+    formats : list
+        The formats to be applied to the Series. Each element in the Series
+        will be assigned a format from this list with the corresponding index.
+    widths: int
+        The number of columns the Series should span. For values higher than 1,
+        the columns spanned will be merged.
+    column_widths : list
+        list of floats representing the column widths of each column within the
+        DataFrame.  If omitted, then widths are set by inspecting the data.
+    row_heights : list
+        list of floats representing the row heights of each row within the
+        Series.  If omitted, then heights are set by inspecting the data.
+    """
+
+    def __init__(self, data, formats=None, width=1, column_widths=1,
+                 row_heights=None, *args, **kwargs):
+        formats = formats or []
+        if type(data) == pd.Series:
+            data = data.to_frame()
+        else:
+            data = pd.Series(data).to_frame()
+        super().__init__(data, formats, width, column_widths, row_heights,
+                         *args, **kwargs)
+
+    def _default_format(self):
+        base_formats = copy.deepcopy(settings['base_formats'])
         return [base_formats.get(
                     str(self.data[0].dtype), base_formats['object'])
                 ] * len(self.data)
 
-class Image:
+
+
+class Image(_XLCBase):
     """ Image allows for the embedding of images into a spreadsheet
 
 
     Parameters
     ----------
     data : str
-        path to the image file, e.g. sample.png or ./sample.jpg
+        path to the image file, e.g. sample.png or ./sample.jpg or a matplotlib
+        `AxesSubplot`
     width : int
         the number of columns consumed by the image
     height : int
@@ -354,6 +459,7 @@ class Image:
     formats : dict
         xlsxwriter options for modifying the image
     """
+
     def __init__(self, data, width=1, height=1, formats={}, *args, **kwargs):
         if data.__class__.__name__=='AxesSubplot':
             inch_to_row = 0.01431127
@@ -372,31 +478,20 @@ class Image:
             self.column_widths = kwargs.get('column_widths')
         self.column_widths = [8.09]*width
 
-    def to_excel(self, workbook_path, default_formats=None):
-        """ Outputs object to Excel.
 
-        Parameters:
-        -----------
-        workbook_path : str
-            The target path and filename of the Excel document
-        """
-        _Workbook(workbook_path=workbook_path, exhibits=self,
-                  default_formats=default_formats).to_excel()
-
-
-class DataFrame:
+class DataFrame(_XLCBase):
     """
-    Excel-ready DataFrame
+    An Excel-ready DataFrame.
 
     Parameters:
     -----------
-    data : DataFrame or Triangle (2D)
-        The data to be places in the exhibit
+    data : DataFrame
+        The data to be placed in the exhibit. Must be a pandas DataFrame or an
+        object with the `to_frame()` method.
     formats : dict
-        The formats to be applied to the data.  Options include
-        'money', 'percent', 'decimal', 'date', 'int', and 'text'.  Each
-        format can be overriden at the class level by overriding its:
-        resepective format dict (e.g. DataFrame.money_format, ...)
+        The formats to be applied to the data columns.  Dictionary keys can be
+        either column names to do column specific formatting OR `xlsxwriter`
+        format names to use consistent formating across the entire `DataFrame`.
     header : bool or list (len of data.columns)
         False uses no headers, True uses headers from data. Alternatively,
         a list of strings will override headers.
@@ -406,31 +501,27 @@ class DataFrame:
         Set to True will insert column numbers into the exhibit.
     index : bool, default True
         Write row names (index).
-    index_formats : dict
-        The formats to be applied to the index, if any
     index_label : str or sequence, optional
         Column label for index column(s) if desired.
-    title : list
-        A list of strings up to length 4 (Title, subtitle1, subtitle2,
-        subtitle3) to be placed above the data in the exhibit.
-
+    index_formats : dict
+        The formats to be applied to the index, if any
+    column_widths : list
+        list of floats representing the column widths of each column within the
+        DataFrame.  If omitted, then widths are set by inspecting the data.
+    row_heights : list
+        list of floats representing the row heights of each row within the
+        DataFrame.  If omitted, then heights are set by inspecting the data.
     """
 
-    min_numeric_col_width = 12
-    # Padding since bold characters are slightly larger than regular
-    # and need a bit more width
-    col_padding_multiplier = 1.1
+    index_formats = copy.deepcopy(settings['index_formats'])
+    header_formats = copy.deepcopy(settings['header_formats'])
+    base_formats = copy.deepcopy(settings['base_formats'])
 
     def __init__(self, data, formats=None,
                  header=True, header_formats=None, col_nums=False,
                  index=True, index_label='', index_formats=None,
-                 title=None, column_widths=None, *args, **kwargs):
-        self.index_formats = {
-            'num_format': '0;(0)', 'text_wrap': True,
-            'bold': True, 'valign': 'bottom', 'align': 'center'}
-        self.header_formats = {
-            'num_format': '0;(0)', 'text_wrap': True, 'bottom': 1,
-            'bold': True, 'valign': 'bottom', 'align': 'center'}
+                 column_widths=None, row_heights=None, *args, **kwargs):
+
         if type(data) is not pd.DataFrame:
             data = data.to_frame()
         self.data = data
@@ -438,40 +529,22 @@ class DataFrame:
         self.index = index
         self.index_label = index_label
         self.col_nums = col_nums
-        self.format_validation(formats)
+        self._format_validation(formats)
         if column_widths is None:
-            self.column_widths = self.get_column_widths()
+            self.column_widths = self._get_column_widths()
         else:
             self.column_widths = column_widths
         self.height = data.shape[0] + self.col_nums + self.header
         self.width = data.shape[1] + self.index
-        if type(title) is str:
-            title = [title]
-        if title is None or title == []:
-            title = None
-        else:
-            self.height = self.height + len(title)
-        if type(title) is list:
-            title = Title(title)
-        self.title = title
         if header_formats is not None:
             self.header_formats.update(header_formats)
         if index_formats is not None:
             self.index_formats.update(index_formats)
+        if row_heights is not None:
+            self._row_heights = row_heights
         self.kwargs = kwargs
 
-    def to_excel(self, workbook_path, default_formats=None):
-        """ Outputs object to Excel.
-
-        Parameters:
-        -----------
-        workbook_path : str
-            The target path and filename of the Excel document
-        """
-        _Workbook(workbook_path=workbook_path, exhibits=self,
-                  default_formats=default_formats).to_excel()
-
-    def get_column_widths(self):
+    def _get_column_widths(self):
         """ Default column widths """
         if self.index:
             row_w = [max(self.data.index.astype(str).str.len())]
@@ -486,23 +559,29 @@ class DataFrame:
                     for item in headers]
         numeric_cols = self.data.select_dtypes('number').columns
         row_w = row_w + \
-                [(self.min_numeric_col_width if item in numeric_cols
+                [(settings['min_numeric_col_width'] if item in numeric_cols
                  else max(self.data[item].astype(str).str.len()))
                  for item in headers]
-        return [max(item)* self.col_padding_multiplier
+        return [max(item)* settings['col_padding_multiplier']
                 for item in zip(header_w, row_w)]
 
-    def format_validation(self, formats):
-        ''' Creates an Excel format compatible dictionary '''
-        base_formats = {
-            'float64': {'num_format': '#,0.00', 'align': 'center'},
-            'float32': {'num_format': '#,0.00', 'align': 'center'},
-            'int64': {'num_format': '#,0', 'align': 'center'},
-            'int32': {'num_format': '#,0', 'align': 'center'},
-            '<M8[ns]': {'num_format': 'yyyy-mm-dd hh:mm', 'align': 'center'},
-            'datetime64[ns]': {'num_format': 'yyyy-mm-dd hh:mm', 'align': 'center'},
-            'object': {'align': 'left'},
-        }
+    @property
+    def row_heights(self):
+        if hasattr(self, '_row_heights'):
+            return self._row_heights
+        return [None]*(len(self.data) + (1 - (not self.header)) + self.col_nums)
+
+    @row_heights.setter
+    def row_heights(self, value):
+        self._row_heights = value
+
+
+    def _format_validation(self, formats):
+        ''' Creates an Excel format compatible dictionary.  Users can specify
+        formats in a variety of shorthand ways and this conforms them to an
+        xlsxwriter style.
+        '''
+
         if self.data.columns.name is not None:
             idx = self.data.index.to_frame().dtypes
             idx.index = [self.data.columns.name]
@@ -510,10 +589,8 @@ class DataFrame:
             idx = pd.Series()
         cols = self.data.dtypes.append(idx)
         self.formats = {
-            k: base_formats.get(v, base_formats['object'])
-            for k, v in dict(cols).items()
-        }
-
+            k: self.base_formats.get(v, self.base_formats['object'])
+            for k, v in dict(cols).items()}
         if type(formats) is list:
             self.formats.update(dict(zip(self.data.columns, formats)))
         elif type(formats) is str:
@@ -521,20 +598,35 @@ class DataFrame:
                 self.data.columns,
                 [{'num_format': formats}] * len(self.data.columns))))
         elif type(formats) is dict and formats != {}:
+            available_formats = [
+                item[4:] for item in dir(xlsxwriter.format.Format)
+                if item[:3]=='set']
             if len(set(self.data.columns).intersection(formats.keys()))==0:
                 self.formats.update(dict(zip(
                     self.data.columns,
                     [formats] * len(self.data.columns))))
-            else:
+            elif len(set(available_formats).intersection(formats.keys()))==0:
                 formats = {k: v if type(v) is dict else {'num_format': v}
                            for k, v in formats.items()}
                 self.formats.update(formats)
+            else:
+                raise AttributeError(
+                    'DataFrame.formats must be a dict with keys set to column names or xlsxwriter formats.')
         else:
             pass
 
 
 class RSpacer(DataFrame):
-    """ Convenience class to create a vertical spacer in a Row container"""
+    """ A blank vertical space in a Row container.
+
+    Parameters
+    ----------
+    width: int
+        Width (in number of columns) of the RSpacer
+    column_widths: float
+        The width to apply to each column of the RSpacer
+    """
+
     def __init__(self, width=1, column_widths=2.25, *args, **kwargs):
         data = pd.DataFrame(dict(zip(list(range(width)), [' '] * width)),
                             index=[0])
@@ -542,30 +634,42 @@ class RSpacer(DataFrame):
         for k, v in temp.__dict__.items():
             setattr(self, k, v)
         self.column_widths = [column_widths] * width
+        self.row_heights = [None]
         self.kwargs = kwargs
 
-
-class VSpacer(RSpacer):
-    pass
+    def _get_html(self, my_height=0, my_width=100):
+        return '<div class="xlccontainer-Spacer" style="height:' + \
+                str(my_height*self.px_per_row) + 'px;width:'+ \
+                str(my_width)+'%;"><div class="xlclabel-Spacer">RSpacer</div></div>\n'
 
 
 class CSpacer(DataFrame):
-    """ Convenience class to create a horizontal spacer in a Column container"""
-    def __init__(self, height=1, column_widths=2.25, *args, **kwargs):
+    """ A blank horizontal space in a Column container.
+
+    Parameters
+    ----------
+    height: int
+        Height (in number of rows) of the CSpacer
+    row_heights: float
+        The height to apply to each row of the CSpacer
+    """
+
+    def __init__(self, height=1, row_heights=None, *args, **kwargs):
         data = pd.DataFrame({' ': [' '] * height})
         temp = DataFrame(data, index=False, header=False)
         for k, v in temp.__dict__.items():
             setattr(self, k, v)
-        self.column_widths = [column_widths]
+        self.column_widths = [2.25]
+        self.row_heights = [row_heights] * height
         self.kwargs = kwargs
 
+    def _get_html(self, my_height=0, my_width=100):
+        return '<div class="xlccontainer-Spacer" style="width: auto;"><div class="xlclabel-Spacer">CSpacer</div></div>\n'
 
-class HSpacer(CSpacer):
-    pass
 
-class _Container():
-    """ Base class for Row and Column
-    """
+class _Container(_XLCBase):
+    """ Base class for Row and Column """
+
     def __init__(self, *args, **kwargs):
         self.args = tuple([copy.deepcopy(item) for item in args])
         self._title_len = 0
@@ -581,31 +685,16 @@ class _Container():
     def __len__(self):
         return len(self.args)
 
-    def to_excel(self, workbook_path, default_formats=None):
-        """ Outputs object to Excel.
-
-        Parameters:
-        -----------
-        workbook_path : str
-            The target path and filename of the Excel document
-        """
-        _Workbook(workbook_path=workbook_path, exhibits=self,
-                  default_formats=default_formats).to_excel()
-
 
 class Row(_Container):
     """
-    Lay out child components in a single horizontal row.
-    Children can be specified as positional arguments, as a single argument
-    that is a sequence.
+    A container object respresenting a horizontal layout of other `xlcompose`
+    objects.
 
     Parameters
     ----------
     args:
-        Children can be of the chainlader DataFrame, Row, and Column classes.
-    title: optional (str or list)
-        The title to be displayed across the top of the container.  Must be
-        specified using the keyword `title=`
+        Any xlcompose objects with the exception of `Sheet` and `Tabs`
 
     Attributes
     ----------
@@ -619,7 +708,7 @@ class Row(_Container):
         super().__init__(*args, **kwargs)
         for num, item in enumerate(self.args):
             if item.__class__.__name__ in ['Title']:
-                self.args = Column(item, Row(*self.args[num + 1:])),
+                self.args = Column(item, Row(*self.args[num + 1:]))
 
     @property
     def height(self):
@@ -643,20 +732,48 @@ class Row(_Container):
     def column_widths(self, value):
         self._column_widths = value
 
+    @property
+    def row_heights(self):
+        if hasattr(self, '_row_heights'):
+            return self._row_heights
+        data = np.array(
+            [item.row_heights for item in self.args
+             if item.__class__.__name__ not in ['Title']])
+        lens = np.array([len(i) for i in data])
+        mask = np.arange(lens.max()) < lens[:,None]
+        out = np.zeros(mask.shape, dtype=data.dtype)
+        out[mask] = np.concatenate(data)
+        out[out==None]=0
+        return [item if item!=0 else None for item in np.max(out, axis=0)]
+
+    @row_heights.setter
+    def row_heights(self, value):
+        self._row_heights = value
+
+    def _get_html(self, my_height=100, my_width=100):
+        direction = ''
+        widths = [item.width/self.width*100 for item in self.args]
+        heights = [item.height for item in self.args]
+        contents = []
+        for num in range(len(self.args)):
+            if self.args[num].__class__.__name__ == 'RSpacer':
+                contents.append(self.args[num]._get_html(max(heights), widths[num]))
+            else:
+                contents.append(self.args[num]._get_html(heights[num], widths[num]))
+        contents = ''.join(contents)
+        width = 'width:' + str(my_width) + '%;' if my_width < 100 else 'width: auto;'
+        return '<div class="xlccontainer-Container" style="' + width + '"><div class="xlclabel-Container">Row</div>' + contents + '</div>\n'
+
 
 class Column(_Container):
     """
-    Lay out child components in a single vertical column.
-    Children can be specified as positional arguments, as a single argument
-    that is a sequence.
+    A container object respresenting a vertical layout of other `xlcompose`
+    objects.
 
     Parameters
     ----------
     args:
-        Children can be of the chainlader DataFrame, Row, and Column classes.
-    title: optional (str or list)
-        The title to be displayed across the top of the container.  Must be
-        specified using the keyword `title=`
+        Any xlcompose objects with the exception of `Sheet` and `Tabs`
 
     Attributes
     ----------
@@ -676,7 +793,6 @@ class Column(_Container):
     def height(self):
         return sum([item.height for item in self.args])
 
-
     @property
     def width(self):
         return max([item.width for item in self.args
@@ -699,17 +815,43 @@ class Column(_Container):
     def column_widths(self, value):
         self._column_widths = value
 
-class Tabs:
+    @property
+    def row_heights(self):
+        if hasattr(self, '_row_heights'):
+            return self._row_heights
+        row_heights = []
+        for item in [getattr(item, 'row_heights', []) for item in self.args]:
+            row_heights = row_heights + item
+        return row_heights
+
+    @row_heights.setter
+    def row_heights(self, value):
+        self._row_heights = row_heights
+
+    def _get_html(self, my_height=100, my_width=100):
+        direction = ''
+        widths = [item.width/self.width*100 for item in self.args]
+        heights = [100]*len(widths)
+        contents = []
+        for num in range(len(self.args)):
+            contents.append(self.args[num]._get_html(heights[num], widths[num]))
+        contents = ''.join(contents)
+        width = 'width:' + str(my_width) + '%;' if my_width < 100 else 'width: auto;'
+        return '<div class="xlccontainer-Container" style="flex-direction: column;' + \
+                width + ';"><div class="xlclabel-Container">Column</div>' + \
+                contents + '</div>\n'
+
+class Tabs(_XLCBase):
     """
-    Layout exhibits across worksheets.
+    A container object representing multiple worksheets.
 
     Parameters
     ----------
     args:
-        Children must be a tuple with a sheet name and any of chainlader
-        DataFrame, Row, Column, Image, or Sheet classes.  For example,
-        ('sheet1', cl.DataFrame(data))
+        A list of sheets or a tuple with a sheet name and any xlcompose object.
+        For example, `('sheet1', xlc.DataFrame(data))`
     """
+    _repr_html_ = None
 
     def __init__(self, *args, **kwargs):
         self.args = [
@@ -725,42 +867,75 @@ class Tabs:
     def __len__(self):
         return len(self.args)
 
-    def to_excel(self, workbook_path, default_formats=None):
-        """ Outputs object to Excel.
 
-        Parameters:
-        -----------
-        workbook_path : str
-            The target path and filename of the Excel document
-
-        """
-        _Workbook(workbook_path=workbook_path, exhibits=self,
-                  default_formats=default_formats).to_excel()
-
-class Sheet:
+class Sheet(_XLCBase):
     """
-    Work with sheet properties.
+    A container object representing an Excel worksheet.
 
     Parameters
     ----------
-    data:
+    name : str
+        The name of the worksheet
+    layout :
         An xlcompose object
+    fit_to_pages:
+        refer to `xlsxwriter` for `fit_to_pages` options
+    freeze_panes:
+        refer to `xlsxwriter` for `freeze_panes` options
+    set_page_view:
+        refer to `xlsxwriter` for `set_page_view` options
+    print_row_col_headers:
+        refer to `xlsxwriter` for `print_row_col_headers` options
+    hide_row_col_headers:
+        refer to `xlsxwriter` for `hide_row_col_headers` options
+    center_vertically:
+        refer to `xlsxwriter` for `center_vertically` options
+    center_horizontally:
+        refer to `xlsxwriter` for `center_horizontally` options
+    set_header:
+        refer to `xlsxwriter` for `set_header` options
+    set_footer:
+        refer to `xlsxwriter` for `set_footer` options
+    repeat_rows:
+        refer to `xlsxwriter` for `repeat_rows` options
+    repeat_columns:
+        refer to `xlsxwriter` for `repeat_columns` options
+    set_margins:
+        refer to `xlsxwriter` for `set_margins` options
+    hide_gridlines:
+        refer to `xlsxwriter` for `hide_gridlines` options
+    set_print_scale:
+        refer to `xlsxwriter` for `set_print_scale` options
+    set_start_page:
+        refer to `xlsxwriter` for `set_start_page` options
+    set_h_pagebreaks:
+        refer to `xlsxwriter` for `set_h_pagebreaks` options
+    set_v_pagebreaks:
+        refer to `xlsxwriter` for `set_v_pagebreaks` options
+    print_across:
+        refer to `xlsxwriter` for `print_across` options
+    print_area:
+        refer to `xlsxwriter` for `print_area` options
+    set_paper:
+        refer to `xlsxwriter` for `set_paper` options
+    set_landscape:
+        refer to `xlsxwriter` for `set_landscape` options
+    set_portrait:
+        refer to `xlsxwriter` for `set_portrait` options
     """
     def __init__(self, name, layout, **kwargs):
         self.name = name
         self.kwargs = kwargs
         self.layout = layout
         self.kwargs = kwargs
-        # Allow for overrides
         self.column_widths = self.layout.column_widths
+        self.row_heights = self.layout.row_heights
 
-    def to_excel(self, workbook_path, default_formats=None):
-        """ Outputs object to Excel.
+    def _repr_html_(self):
+        return self.layout._repr_html_()
 
-        Parameters:
-        -----------
-        workbook_path : str
-            The target path and filename of the Excel document
-        """
-        _Workbook(workbook_path=workbook_path, exhibits=self,
-                  default_formats=default_formats).to_excel()
+class VSpacer(RSpacer):
+    pass
+
+class HSpacer(CSpacer):
+    pass
